//  Copyright (c) 2005-2006 Stephan Hirmer (shirmer@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(BOOST_PP_IS_ITERATING)

#if !defined(SAGA_ADAPTOR_TASK_IMPL_HPP)
#define SAGA_ADAPTOR_TASK_IMPL_HPP

#include <impl/config.hpp>

#include <boost/preprocessor/iterate.hpp>
#include <boost/preprocessor/repetition/enum_params.hpp>
#include <boost/preprocessor/repetition/enum_binary_params.hpp>

#define BOOST_PP_ITERATION_PARAMS_1                                           \
    (3, (1, SAGA_ARGUMENT_LIMIT,                                              \
    "saga/adaptors/task_impl.hpp"))                                           \
    /**/

#include BOOST_PP_ITERATE()

#endif // SAGA_ADAPTOR_TASK_IMPL_HPP

///////////////////////////////////////////////////////////////////////////////
//
//  Preprocessor vertical repetition code
//
///////////////////////////////////////////////////////////////////////////////
#else // defined(BOOST_PP_IS_ITERATING)

#define P BOOST_PP_ITERATION()

#define GET_TEMP_ARG(z, M, _) BOOST_PP_COMMA_IF(M) FuncArg ## M, Arg ## M

    private:
    
    
    /**
      * templated helper function, which creates a appropriate
      * saga::impl::task object
      */
    template<
        typename Cpi, 
        typename FuncBase,
        typename FuncRetV, 
        typename RetVal,
        BOOST_PP_ENUM_PARAMS (P, typename FuncArg),
        BOOST_PP_ENUM_PARAMS (P, typename Arg)
    >
    static saga::impl::task <Cpi, 
                             FuncBase,
                             FuncRetV, RetVal,
                             BOOST_PP_REPEAT(P, GET_TEMP_ARG, _)
                            >*
    create_task(char const *func_name, 
                TR1::shared_ptr<Cpi> cpi_, 
                void (FuncBase::*f)(FuncRetV, BOOST_PP_ENUM_PARAMS(P, FuncArg) ),
                RetVal func_ret, 
                BOOST_PP_ENUM_BINARY_PARAMS (P, Arg, arg),
                bool (FuncBase::*prep_func)(FuncRetV,
                                            BOOST_PP_ENUM_PARAMS(P, FuncArg), saga::uuid) = NULL)
    {
        return new saga::impl::task<Cpi, FuncBase,
                                    FuncRetV, RetVal,
                                    BOOST_PP_REPEAT(P, GET_TEMP_ARG, _) 
                                   >
                                    (func_name,
                                     cpi_, f, func_ret,
                                     BOOST_PP_ENUM_PARAMS(P, arg),
                                     prep_func);
    }
    
    
    /**
      * templated helper function, which creates a appropriate
      * saga::impl::task object
      *
      * The function to execute takes as a further parameter,
      * more precisely, a uuid of the task, which will be executed in the adaptor
      *
      * generated by the preprocessor with the help of macros. The number of
      * parameters differs.
      */
    template<
        typename Cpi, 
        typename FuncBase,
        typename FuncRetV, 
        typename RetVal,
        BOOST_PP_ENUM_PARAMS (P, typename FuncArg),
        BOOST_PP_ENUM_PARAMS (P, typename Arg)
    >
    static saga::impl::wrapper_task <Cpi, 
                                      FuncBase,
                                      FuncRetV, RetVal,
                                      BOOST_PP_REPEAT(P, GET_TEMP_ARG, _)
                                      >*
    create_task(char const *func_name, 
                TR1::shared_ptr<Cpi> cpi_, 
                void (FuncBase::*f)(FuncRetV, BOOST_PP_ENUM_PARAMS(P, FuncArg), saga::uuid ),
                RetVal func_ret, 
                BOOST_PP_ENUM_BINARY_PARAMS (P, Arg, arg),
                bool (FuncBase::*prep_func)(FuncRetV,
                                            BOOST_PP_ENUM_PARAMS(P, FuncArg), saga::uuid) = NULL)
    {
        return new saga::impl::wrapper_task<Cpi, FuncBase,
                                            FuncRetV, RetVal,
                                            BOOST_PP_REPEAT(P, GET_TEMP_ARG, _) 
                                           >
                                            (func_name,
                                             cpi_, f, func_ret,
                                             BOOST_PP_ENUM_PARAMS(P, arg),
                                             prep_func);
    }

#undef GET_TEMP_ARG

    public:
    
    /**
      * overloaded templated constructor
      *
      * generates with the help of preprocessor meta programming.
      * Same as the simple constructor, but the number of function parameters
      * for f and prep_func differ.
      * calls member function create_task and puts the returned task
      * into the saga::task class, which is part of the facade.
      *
      */ 
    template<typename Cpi,
                typename BaseCpi,
                typename FuncRetV,
                typename RetVal,
                BOOST_PP_ENUM_PARAMS (P, typename FuncArg),
                BOOST_PP_ENUM_PARAMS (P, typename Arg)>
    task (  char const *func_name,
            TR1::shared_ptr<Cpi> cpi_,
            void (BaseCpi::*func) (FuncRetV, 
                                   BOOST_PP_ENUM_PARAMS (P, FuncArg) ),
            RetVal func_ret,
            BOOST_PP_ENUM_BINARY_PARAMS (P, Arg, arg),
            bool (BaseCpi::*prep_func) (FuncRetV, 
                                        BOOST_PP_ENUM_PARAMS (P, FuncArg), saga::uuid) = NULL)         
      : base_type (create_task (func_name, cpi_, 
                              func, func_ret, 
                              BOOST_PP_ENUM_PARAMS(P, arg), 
                              prep_func ))
    {                
    }
    
    /**
      * overloaded templated constructor
      *
      * generates with the help of preprocessor metaprogramming.
      * Same as the simple constructor, but the number of function parameters
      * for f and prep_func differ.
      * calls memeber function create_task and puts the returned task
      * into the saga::task class, which is part of the facade.
      * Differs from the other constructor, because the func takes a 
      * further parameter, a task uuid.
      *
      */ 
    template<typename Cpi,
                typename BaseCpi,
                typename FuncRetV,
                typename RetVal,
                BOOST_PP_ENUM_PARAMS (P, typename FuncArg),
                BOOST_PP_ENUM_PARAMS (P, typename Arg)>
    task (  char const *func_name,
            TR1::shared_ptr<Cpi> cpi_,
            void (BaseCpi::*func) (FuncRetV, 
                                   BOOST_PP_ENUM_PARAMS (P, FuncArg), saga::uuid ),
            RetVal func_ret,
            BOOST_PP_ENUM_BINARY_PARAMS (P, Arg, arg),
            bool (BaseCpi::*prep_func) (FuncRetV, 
                                        BOOST_PP_ENUM_PARAMS (P, FuncArg), saga::uuid) = NULL)         
      : base_type(create_task(func_name, cpi_, func, func_ret, 
                              BOOST_PP_ENUM_PARAMS(P, arg), prep_func )) 
    {                
    }

#undef P

#endif // defined(BOOST_PP_IS_ITERATING)
