//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_CALL_V1_HPP
#define SAGA_CALL_V1_HPP

// include macros for private functions and implementation
#include <saga/detail/call_v1.hpp>   

///////////////////////////////////////////////////////////////////////////////
//
// FIXME: document usage extensively
//

///////////////////////////////////////////////////////////////////////////////
//
// 0 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_0_DEF_0(rt, name)                                       \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret)                                         \
      { return  (name ## priv (ret, Tag())); }                                \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_0_DEF_0(rt, name)                                 \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret) const                                   \
      { return  (name ## priv (ret, Tag())); }                                \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 1 parameter
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_1_DEF_0(rt, name, p1)                                   \
       template <typename Tag>                                                \
      saga::task_type name (rt & ret, p1 par1)                                \
      { return  (name ## priv (ret, par1, Tag())); }                          \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_1_DEF_1(rt, name, p1, d1)                               \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1)                           \
      { return  (name ## priv (ret, par1, Tag())); }                          \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_1_DEF_0(rt, name, p1)                             \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1) const                          \
      { return  (name ## priv (ret, par1, Tag())); }                          \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_1_DEF_1(rt, name, p1, d1)                         \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1) const                     \
      { return  (name ## priv (ret, par1, Tag())); }                          \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 2 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_2_DEF_0(rt, name, p1, p2)                               \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2)                       \
      { return  (name ## priv (ret, par1, par2, Tag())); }                    \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_2_DEF_1(rt, name, p1, p2, d2)                           \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2)                  \
      { return  (name ## priv (ret, par1, par2, Tag())); }                    \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_2_DEF_2(rt, name, p1, d1, p2, d2)                       \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2)             \
      { return  (name ## priv (ret, par1, par2, Tag())); }                    \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_2_DEF_0(rt, name, p1, p2)                         \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2) const                 \
      { return  (name ## priv (ret, par1, par2, Tag())); }                    \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_2_DEF_1(rt, name, p1, p2, d2)                     \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2) const            \
      { return  (name ## priv (ret, par1, par2, Tag())); }                    \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_2_DEF_2(rt, name, p1, d1, p2, d2)                 \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2) const       \
      { return  (name ## priv (ret, par1, par2, Tag())); }                    \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 3 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_3_DEF_0(rt, name, p1, p2, p3)                           \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3)              \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_3_DEF_1(rt, name, p1, p2, p3, d3)                       \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3 = d3)         \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_3_DEF_2(rt, name, p1, p2, d2, p3, d3)                   \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2, p3 par3 = d3)    \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_3_DEF_3(rt, name, p1, d1, p2, d2, p3, d3)               \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2, p3 par3 = d3) \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_3_DEF_0(rt, name, p1, p2, p3)                     \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3) const        \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_3_DEF_1(rt, name, p1, p2, p3, d3)                 \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3 = d3) const   \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_3_DEF_2(rt, name, p1, p2, d2, p3, d3)             \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2, p3 par3 = d3) const \
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_3_DEF_3(rt, name, p1, d1, p2, d2, p3, d3)         \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2, p3 par3 = d3) const\
      { return  (name ## priv (ret, par1, par2, par3, Tag())); }              \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 4 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_4_DEF_0(rt, name, p1, p2, p3, p4)                       \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4)     \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_4_DEF_1(rt, name, p1, p2, p3, p4, d4)                   \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4 = d4)\
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_4_DEF_2(rt, name, p1, p2, p3, d3, p4, d4)               \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3 = d3, p4 par4 = d4)\
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_4_DEF_3(rt, name, p1, p2, d2, p3, d3, p4, d4)           \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2, p3 par3 = d3,    \
                       p4 par4 = d4)                                          \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_4_DEF_4(rt, name, p1, d1, p2, d2, p3, d3, p4, d4)       \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2, p3 par3 = d3, \
                       p4 par4 = d4)                                          \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_4_DEF_0(rt, name, p1, p2, p3, p4)                 \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4) const \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_4_DEF_1(rt, name, p1, p2, p3, p4, d4)             \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4 = d4) const \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_4_DEF_2(rt, name, p1, p2, p3, d3, p4, d4)         \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3 = d3, p4 par4 = d4) const \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_4_DEF_3(rt, name, p1, p2, d2, p3, d3, p4, d4)     \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2, p3 par3 = d3,    \
                       p4 par4 = d4) const                                    \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_4_DEF_4(rt, name, p1, d1, p2, d2, p3, d3, p4, d4) \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2, p3 par3 = d3, \
                       p4 par4 = d4) const                                    \
      { return  (name ## priv (ret, par1, par2, par3, par4, Tag())); }        \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 5 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_5_DEF_0(rt, name, p1, p2, p3, p4, p5)                   \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, p5 par5) \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_5_DEF_1(rt, name, p1, p2, p3, p4, p5, d5)               \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4,     \
                       p5 par5 = d5)                                          \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_5_DEF_2(rt, name, p1, p2, p3, p4, d4, p5, d5)           \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4 = d4,\
                       p5 par5 = d5)                                          \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_5_DEF_3(rt, name, p1, p2, p3, d3, p4, d4, p5, d5)       \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3 = d3,         \
                       p4 par4 = d4, p5 par5 = d5)                            \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_5_DEF_4(rt, name, p1, p2, d2, p3, d3, p4, d4, p5, d5)   \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2, p3 par3 = d3,    \
                       p4 par4 = d4, p5 par5 = d5)                            \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_PUB_5_DEF_5(rt, name, p1, d1, p2, d2, p3, d3, p4, d4, p5, d5) \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2, p3 par3 = d3, \
                       p4 par4 = d4, p5 par5 = d5)                            \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_5_DEF_0(rt, name, p1, p2, p3, p4, p5)             \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p5 par5) const \
      { return  (name ## priv (ret, par1, par2, par3, par5, Tag())); }        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_5_DEF_1(rt, name, p1, p2, p3, p4, p5, d5)         \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4,     \
                       p5 par5 = d5) const                                    \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_5_DEF_2(rt, name, p1, p2, p3, p4, d4, p5, d5)     \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4 = d4,\
                       p5 par5 = d5) const                                    \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_5_DEF_3(rt, name, p1, p2, p3, d3, p4, d4, p5, d5) \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2, p3 par3 = d3, p4 par4 = d4,\
                       p5 par5 = d5) const                                    \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_5_DEF_4(rt, name, p1, p2, d2, p3, d3, p4, d4, p5, d5) \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1, p2 par2 = d2, p3 par3 = d3,    \
                       p4 par4 = d4, p5 par5 = d5) const                      \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PUB_5_DEF_5(rt, name, p1, d1, p2, d2, p3, d3, p4, d4, p5, d5) \
      template <typename Tag>                                                 \
      saga::task_type name (rt & ret, p1 par1 = d1, p2 par2 = d2, p3 par3 = d3, \
                       p4 par4 = d4, p5 par5 = d5) const                      \
      { return  (name ## priv (ret, par1, par2, par3, par4, par5, Tag())); }  \
//-----------------------------------------------------------------------------

#endif // SAGA_CALL_V1_HPP

