//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_CONTEXT_HPP
#define SAGA_CONTEXT_HPP

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/context_base.hpp>
#include <saga/task.hpp>

#include <saga/object.hpp>
#include <saga/detail/attribute.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
//
//  SAGA spec:
//
//  package saga.context
//  {
//    enum context_type 
//    {
//      Unknown         = -1,
//      Any             =  0,
//      X509            =  1, // Globus
//      MyProxy         =  2, // MyProxy 
//      SAML            =  3, // SAML
//      Unicore         =  4  // Unicore
//      SSH             =  5, // SSH
//      Kerberos        =  6, // Kerberos
//      UserPass        =  7  // FTP etc.
//    }
// 
//    class context : implements-all saga::object
//                    implements-all saga::attribute 
//    {  
//      CONSTRUCTOR (in  context_type type,
//                   out context      context);
//      DESTRUCTOR  (in  context      context);
// 
//      get_ctype   (out context_type type); 
//    }
//  }
///////////////////////////////////////////////////////////////////////////////

namespace saga
{
  namespace attributes 
  {
    ///////////////////////////////////////////////////////////////////////////
    //  attribute names for context

    //  read write attributes
    char const* const context_cert_dir    = "CertDir";
    char const* const context_user_proxy  = "UserProxy";
    char const* const context_user_cert   = "UserCert";
    char const* const context_user_name   = "UserName";
    char const* const context_user_key    = "UserKey";
    char const* const context_user_pass   = "UserPass";
    char const* const context_server      = "Server";

    //  read only attributes
    char const* const context_remote_host = "RemoteHost";
    char const* const context_remote_id   = "RemoteID";
    char const* const context_remote_port = "RemotePort";
  }
  
  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  class SAGA_EXPORT context
    : public saga::context_base
    , public saga::object
    , public detail::attribute<context>
  {
    protected:
      /// @cond
      /** These methods are not within API scope */
      TR1::shared_ptr <saga::impl::context> get_impl (void) const;
      friend struct detail::attribute<context>;  // needs to access get_impl()
      friend class saga::impl::context;
      
      context (saga::impl::context *impl);
      /// @endcond
      
    public:

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      typedef context_base::type type;

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      explicit context   (type t = saga::context::Unknown);
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      context (saga::object const& o);
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
     ~context   (void);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      context &operator= (saga::object const& o);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      type get_ctype (void) const;

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      friend SAGA_EXPORT 
      bool operator== (context const & lhs, context const & rhs);

  }; 

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_CONTEXT_HPP

