//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_DETAIL_CALL_V1_HPP
#define SAGA_DETAIL_CALL_V1_HPP

///////////////////////////////////////////////////////////////////////////////
//
// FIXME: document usage extensively
//
//  This file defines the API macros for the private functions and the 
//  function implementation.

///////////////////////////////////////////////////////////////////////////////
//
// 0 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PRIV_0(rt, name)                                            \
      saga::task_type name ## priv (rt &, saga::task_base::Sync) ;            \
      saga::task_type name ## priv (rt &, saga::task_base::ASync);            \
      saga::task_type name ## priv (rt &, saga::task_base::Task) ;            \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PRIV_0(rt, name)                                      \
      saga::task_type name ## priv (rt &, saga::task_base::Sync)  const;      \
      saga::task_type name ## priv (rt &, saga::task_base::ASync) const;      \
      saga::task_type name ## priv (rt &, saga::task_base::Task)  const;      \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_IMP_0(ns, rt, name)                                         \
  saga::task_type ns::name ## priv (rt & ret, saga::task_base::Sync)          \
  { return            (get_impl()->name (ret, true)); }                       \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, saga::task_base::ASync)         \
  { return (saga::detail::run (get_impl()->name (ret))); }                            \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, saga::task_base::Task)          \
  { return            (get_impl()->name (ret)); }                             \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_IMP_0(ns, rt, name)                                   \
  saga::task_type ns::name ## priv (rt & ret, saga::task_base::Sync)  const   \
  { return            (get_impl()->name (ret, true)); }                       \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, saga::task_base::ASync) const   \
  { return (saga::detail::run (get_impl()->name (ret))); }                            \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, saga::task_base::Task)  const   \
  { return            (get_impl()->name (ret)); }                             \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 1 parameter
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PRIV_1(rt, name, p1)                                        \
      saga::task_type name ## priv (rt &, p1, saga::task_base::Sync) ;        \
      saga::task_type name ## priv (rt &, p1, saga::task_base::ASync);        \
      saga::task_type name ## priv (rt &, p1, saga::task_base::Task) ;        \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PRIV_1(rt, name, p1)                                  \
      saga::task_type name ## priv (rt &, p1, saga::task_base::Sync)  const;  \
      saga::task_type name ## priv (rt &, p1, saga::task_base::ASync) const;  \
      saga::task_type name ## priv (rt &, p1, saga::task_base::Task)  const;  \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_IMP_1(ns, rt, name, p1)                                     \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, saga::task_base::Sync) \
  { return            (get_impl()->name (ret, par1, true)); }                 \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, saga::task_base::ASync)\
  { return (saga::detail::run (get_impl()->name (ret, par1))); }                      \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, saga::task_base::Task) \
  { return            (get_impl()->name (ret, par1)); }                       \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_IMP_1(ns, rt, name, p1)                               \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, saga::task_base::Sync) const \
  { return            (get_impl()->name (ret, par1, true)); }                 \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, saga::task_base::ASync) const \
  { return (saga::detail::run (get_impl()->name (ret, par1))); }                      \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, saga::task_base::Task) const \
  { return            (get_impl()->name (ret, par1)); }                       \
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//
// 2 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PRIV_2(rt, name, p1, p2)                                    \
      saga::task_type name ## priv (rt &, p1, p2, saga::task_base::Sync) ;    \
      saga::task_type name ## priv (rt &, p1, p2, saga::task_base::ASync);    \
      saga::task_type name ## priv (rt &, p1, p2, saga::task_base::Task) ;    \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PRIV_2(rt, name, p1, p2)                              \
      saga::task_type name ## priv (rt &, p1, p2, saga::task_base::Sync)  const; \
      saga::task_type name ## priv (rt &, p1, p2, saga::task_base::ASync) const; \
      saga::task_type name ## priv (rt &, p1, p2, saga::task_base::Task)  const; \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_IMP_2(ns, rt, name, p1, p2)                                 \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, saga::task_base::Sync) \
  { return            (get_impl()->name (ret, par1, par2, true)); }           \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, saga::task_base::ASync) \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2))); }                \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, saga::task_base::Task) \
  { return            (get_impl()->name (ret, par1, par2)); }                 \
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_IMP_2(ns, rt, name, p1, p2)                           \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, saga::task_base::Sync) const \
  { return            (get_impl()->name (ret, par1, par2, true)); }           \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, saga::task_base::ASync) const \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2))); }                \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, saga::task_base::Task) const \
  { return            (get_impl()->name (ret, par1, par2)); }                 \
//-----------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
//
// 3 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PRIV_3(rt, name, p1, p2, p3)                                \
      saga::task_type name ## priv (rt &, p1, p2, p3, saga::task_base::Sync) ;\
      saga::task_type name ## priv (rt &, p1, p2, p3, saga::task_base::ASync);\
      saga::task_type name ## priv (rt &, p1, p2, p3, saga::task_base::Task) ;\
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PRIV_3(rt, name, p1, p2, p3)                          \
      saga::task_type name ## priv (rt &, p1, p2, p3, saga::task_base::Sync) const; \
      saga::task_type name ## priv (rt &, p1, p2, p3, saga::task_base::ASync) const; \
      saga::task_type name ## priv (rt &, p1, p2, p3, saga::task_base::Task) const; \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_IMP_3(ns, rt, name, p1, p2, p3)                             \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, saga::task_base::Sync) \
  { return            (get_impl()->name (ret, par1, par2, par3, true)); }     \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, saga::task_base::ASync)\
  { return (saga::detail::run (get_impl()->name (ret, par1, par2, par3))); }          \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, saga::task_base::Task) \
  { return            (get_impl()->name (ret, par1, par2, par3)); }           \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_IMP_3(ns, rt, name, p1, p2, p3)                       \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, saga::task_base::Sync) const \
  { return            (get_impl()->name (ret, par1, par2, par3, true)); }     \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, saga::task_base::ASync) const \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2, par3))); }          \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, saga::task_base::Task) const \
  { return            (get_impl()->name (ret, par1, par2, par3)); }           \
//-----------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
//
// 4 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PRIV_4(rt, name, p1, p2, p3, p4)                            \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, saga::task_base::Sync); \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, saga::task_base::ASync); \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, saga::task_base::Task); \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PRIV_4(rt, name, p1, p2, p3, p4)                       \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, saga::task_base::Sync) const; \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, saga::task_base::ASync) const; \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, saga::task_base::Task) const; \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_IMP_4(ns, rt, name, p1, p2, p3, p4)                         \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               saga::task_base::Sync)                         \
  { return            (get_impl()->name (ret, par1, par2, par3, par4, true)); } \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               saga::task_base::ASync)                        \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2, par3, par4))); }    \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               saga::task_base::Task)                         \
  { return            (get_impl()->name (ret, par1, par2, par3, par4)); }     \
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_IMP_4(ns, rt, name, p1, p2, p3, p4)                   \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               saga::task_base::Sync) const                   \
  { return            (get_impl()->name (ret, par1, par2, par3, par4, true)); } \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               saga::task_base::ASync) const                  \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2, par3, par4))); }    \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               saga::task_base::Task) const                   \
  { return            (get_impl()->name (ret, par1, par2, par3, par4)); }     \
//-----------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
//
// 5 parameters
//
///////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
#define SAGA_CALL_PRIV_5(rt, name, p1, p2, p3, p4, p5)                        \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, p5, saga::task_base::Sync); \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, p5, saga::task_base::ASync); \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, p5, saga::task_base::Task); \
//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_PRIV_5(rt, name, p1, p2, p3, p4, p5)                  \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, p5, saga::task_base::Sync) const; \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, p5, saga::task_base::ASync) const; \
      saga::task_type name ## priv (rt &, p1, p2, p3, p4, p5, saga::task_base::Task) const; \
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
#define SAGA_CALL_IMP_5(ns, rt, name, p1, p2, p3, p4, p5)                     \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               p5 par5, saga::task_base::Sync)                \
  { return            (get_impl()->name (ret, par1, par2, par3, par4, par5,   \
                                         true)); }                            \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               p5 par5, saga::task_base::ASync)               \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2, par3, par4, par5))); } \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               p5 par5, saga::task_base::Task)                \
  { return            (get_impl()->name (ret, par1, par2, par3, par4, par5)); } \
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
#define SAGA_CALL_CONST_IMP_5(ns, rt, name, p1, p2, p3, p4, p5)               \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               p5 par5, saga::task_base::Sync) const          \
  { return            (get_impl()->name (ret, par1, par2, par3, par4, par5,   \
                                         true)); }                            \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               p5 par5, saga::task_base::ASync) const         \
  { return (saga::detail::run (get_impl()->name (ret, par1, par2, par3, par4, par5))); } \
                                                                              \
  saga::task_type ns::name ## priv (rt & ret, p1 par1, p2 par2, p3 par3, p4 par4, \
                               p5 par5, saga::task_base::Task) const          \
  { return            (get_impl()->name (ret, par1, par2, par3, par4, par5)); } \
//-----------------------------------------------------------------------------

#endif // SAGA_DETAIL_CALL_V1_HPP

