//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_SAGA_DETAIL_MONITORABLE_HPP
#define SAGA_SAGA_DETAIL_MONITORABLE_HPP

// include stl
#include <vector>
#include <string>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/monitorable.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable : 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
// for modules not belonging to the engine attributes should not be exported
#if !defined (SAGA_CREATE_PREPROCESSED_FILES) && defined (BOOST_WINDOWS) 
#  ifdef     SAGA_ENGINE_EXPORTS
#    define  SAGA_EXPORT_MONITORABLE   __declspec (dllexport) 
#  elif !defined (SAGA_NO_EXPORT_MONITORABLE)
#    define  SAGA_EXPORT_MONITORABLE   __declspec (dllimport) 
#  endif
#endif

#if !defined (SAGA_EXPORT_MONITORABLE)
#    define  SAGA_EXPORT_MONITORABLE   /* empty */
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace detail
    {
        
        /*! \brief   The %monitorable interface.
         *
         *  \details Detailed %description starts here.
         */
        template <typename Derived>
        struct SAGA_EXPORT_MONITORABLE monitorable 
        {
            /*! \brief   A %cookie_handle is a unique identifier for a callback.
             *
             *  \details Detailed %description starts here.
             */
            typedef saga::monitorable::cookie_handle cookie_handle;
            
            ///@cond
            void init (std::vector <saga::metric> const & metrics);
            
            Derived & derived (void) 
            { 
                return static_cast <Derived&> (*this); 
            }
            
            Derived const & derived (void) const 
            { 
                return static_cast <Derived const &> (*this); 
            }
            ///@endcond
            
            /*! \brief     Lists all %metrics assiciated with this %object instance.
             *
             *  \return    A vector of available. %metrics
             *  \exception saga::exception (on condition)
             *
             *  \details   Detailed %description starts here.
             */
            std::vector <saga::metric> list_metrics (void) const;
            
            /*! \brief     Returns a %metric associated with this %object instance.
             *
             *  \param     name The metric's name
             *  \return    The %metric identified by name
             *  \exception saga::exception (on condition)
             *
             *  \details Detailed %description starts here.
             */
            saga::metric get_metric (std::string name) const;
            
            /*! \brief Adds a callback to this %object instance.
             *
             *  \param     name The new callback's name
             *  \param     cb The callback %object
             *  \return    A unique handle to the new callback
             *  \exception saga::exception (on condition)
             *
             *  \details   Detailed %description starts here.
             */
            cookie_handle add_callback (std::string name, saga::callback cb);
            
            /*! \brief     Removes a callback from this %object instance.
             *
             *  \param     name The callback's name
             *  \param     cookie The callback's unique handle
             *  \exception saga::exception (on condition)
             *
             *  \details   Detailed %description starts here.
             */
            void remove_callback (std::string name, cookie_handle cookie);
        };
    } 
} // namespace saga::detail
///////////////////////////////////////////////////////////////////////////////

#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_SAGA_DETAIL_MONITORABLE_HPP

