//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_SAGA_DETAIL_STEERABLE_HPP
#define SAGA_SAGA_DETAIL_STEERABLE_HPP

// include stl
#include <vector>
#include <string>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/monitorable.hpp>
#include <saga/detail/monitorable.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable : 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
// for modules not belonging to the engine attributes should not be exported
#if !defined (SAGA_CREATE_PREPROCESSED_FILES) && defined (BOOST_WINDOWS) 
#  ifdef     SAGA_ENGINE_EXPORTS
#    define  SAGA_EXPORT_STEERABLE   __declspec (dllexport) 
#  elif !defined (SAGA_NO_EXPORT_STEERABLE)
#    define  SAGA_EXPORT_STEERABLE   __declspec (dllimport) 
#  endif
#endif

#if !defined (SAGA_EXPORT_STEERABLE)
#    define  SAGA_EXPORT_STEERABLE   /* empty */
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace detail
    {
        /*! \brief   The %steerable interface.
         *
         *  \details Detailed %description starts here.
         */
        template <typename Derived>
        struct SAGA_EXPORT_STEERABLE steerable
        {
            ///@cond
            Derived& derived() 
            { return static_cast<Derived&>(*this); }
            Derived const& derived() const 
            { return static_cast<Derived const&>(*this); }
            ///@endcond
            
            /*! \brief     Adds a %metric to this %object instance.
             *
             *  \param     m The %metric to add
             *  \exception saga::exception (on condition)
             *
             *  \details Detailed %description starts here.
             */
            bool add_metric (saga::metric m);
            
            /*! \brief     Removes a %metric from this %object instance.
             *
             *  \param     name The metric's name
             *  \exception saga::exception (on condition)
             *
             *  \details Detailed %description starts here.
             */
            void remove_metric (std::string name);

            /*! \brief     Push a new %metric value to the backend.
             *
             *  \param     name The %metric to be fired 
             *  \exception saga::exception (on condition)
             *
             *  \details Detailed %description starts here.
             */
            void fire_metric (std::string name);
        };
        
        ///////////////////////////////////////////////////////////////////////////////
    }}   // namespace saga::detail

#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_SAGA_DETAIL_STEERABLE_HPP

