//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(SAGA_ERROR_HPP)
#define SAGA_ERROR_HPP

///////////////////////////////////////////////////////////////////////////////
namespace saga {

  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  typedef enum error {
    //  This list is sorted, with the most specific exceptions are listed first 
    //  and least specific last.
    //  The most specific exception possible (i.e., applicable) MUST be thrown 
    //  on all error conditions.
    Success               = 0,
    NotImplemented        = 1,
    IncorrectURL          = 2,
    BadParameter          = 3,
    AlreadyExists         = 4,
    DoesNotExist          = 5,
    IncorrectState        = 6,
    PermissionDenied      = 7,
    AuthorizationFailed   = 8,
    AuthenticationFailed  = 9,
    Timeout               = 10,
    // these are not in the SAGA spec!
    // FIXME: then they MUST NOT be exposed on API level - AM
        NoAdaptor         = 11,
        NoAdaptorInfo     = 12,
        Unexpected        = 13,
    NoSuccess             = 14,
  };

  /// @cond
  /** These methods are not within API scope */
  char const* const error_names[] = {
    "Success",
    "NotImplemented",
    "IncorrectURL",
    "BadParameter",
    "AlreadyExists",
    "DoesNotExist",
    "IncorrectState",
    "PermissionDenied",
    "AuthorizationFailed",
    "AuthenticationFailed",
    "Timeout",
    // these are not in the SAGA spec!
    // FIXME: then they MUST NOT be exposed on API level - AM
        "NoAdaptor",
        "NoAdaptorInfo",
        "Unexpected",
    "NoSuccess",
  };
  /// @endcond
  
///////////////////////////////////////////////////////////////////////////////
} // namespace saga

#endif // SAGA_ERROR_HPP

