//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_IMPL_ENGINE_PERMISSIONS_HPP
#define SAGA_IMPL_ENGINE_PERMISSIONS_HPP

#include <string>

#include <impl/call.hpp>
#include <impl/engine/permissions_interface.hpp>
#include <saga/adaptors/permissions_cpi.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable : 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace impl
{
    // in SAGA, permissions is actually an interface. 
    class SAGA_EXPORT permissions : public permissions_interface
    {
    private:
        typedef v1_0::permissions_cpi permissions_cpi;
        typedef v1_0::preference_type preference_type;
        
    public:
        ~permissions() {} 
        
        // setters/getters 
        
        /** saga::void_t permissions_allow(std::string object, int i)
         *  @brief Return the type of the GATJob
         *
         *  The function @c GATJob_GetType always returns 
         *  @c #GATType_GATJob. 
         *
         *  @param object The object to inspect
         *
         *  @return Returns always @c #GATType_GATJob. 
         */
        SAGA_CALL_IMPL_DECL_2(saga::void_t, permissions_allow, std::string, int);
        SAGA_CALL_IMPL_DECL_2(saga::void_t, permissions_deny, std::string, int);
        SAGA_CALL_IMPL_DECL_2(bool, permissions_check, std::string, int);
        SAGA_CALL_IMPL_DECL_0(std::string, get_owner);
        SAGA_CALL_IMPL_DECL_0(std::string, get_group);

        // get_proxy returns the internal implementation of the 
        // impl::object interface
        virtual saga::impl::proxy* get_proxy();
        virtual saga::impl::proxy const* get_proxy() const;
    };

///////////////////////////////////////////////////////////////////////////////
}} // namespace saga::impl

#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif 

