//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(SAGA_IMPL_ENGINE_RUN_MODE_WRAPPER_HPP)
#define SAGA_IMPL_ENGINE_RUN_MODE_WRAPPER_HPP

#include <boost/assert.hpp>
#include <impl/exception_list.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace impl {

    ///////////////////////////////////////////////////////////////////////////
    template <typename FuncBase, typename FuncRetVal, typename RetVal>
    inline task_type 
    execute_sync_async(proxy *prxy,
        char const *cpi_name, char const *name, char const *op_name, 
        v1_0::preference_type const& prefs, bool is_sync,
        void (FuncBase::*sync_func)(FuncRetVal), 
        task_type (FuncBase::*async_func)(FuncRetVal), RetVal ret)
    {
        adaptor_selector::adaptor_info_list_type no_no_list;
        exception_list exceptions;
        bool found_one = false;
        while (true) {
            v1_0::cpi_info info;
            try {
                run_mode current_mode = Unknown;
                cpi_list::cpi_type current_cpi;
                 
                {
                // try to create a cpi_instance implementing the required 
                // function
                    proxy::mutex_type::scoped_lock lock (prxy->mtx_);
                    current_mode = prxy->select_run_mode(cpi_name, name, prefs, 
                        is_sync, no_no_list);
                    BOOST_ASSERT(!prxy->cpis_.empty());
                    
                // retrieve info to be able to store it in no_no_list later
                    current_cpi = prxy->cpis_.get_current();
                    info = current_cpi->get_adaptor_info();
                }
                
                found_one = true;
                return dispatch_sync_async (current_mode, current_cpi, 
                    sync_func, async_func, op_name, ret);
            }
            catch (saga::exception const& e) {
                // save this exception, avoid NoAdaptor errors if we actually 
                // found at least one
                if (!found_one || e.get_error() != saga::NoAdaptor)
                    exceptions.add(e);

                // throw, if we don't find anything                
                if (e.get_error() == saga::NoAdaptor || 
                    e.get_error() == saga::NoAdaptorInfo)
                {
                    SAGA_THROW_VERBATIM(prxy, exceptions.get_message(), 
                        exceptions.get_error());
                }

                // just try the next adaptor
                no_no_list.push_back(info);
            }
        }
    }

    // bring in higher order functions
    #include <impl/engine/run_mode_wrapper_impl.hpp>

///////////////////////////////////////////////////////////////////////////////
}}  // namespace saga::impl

#endif // !SAGA_IMPL_ENGINE_RUN_MODE_WRAPPER_HPP



