//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(BOOST_PP_IS_ITERATING)

#if !defined(SAGA_IMPL_ENGINE_SYNC_ASYNC_IMPL_HPP)
#define SAGA_IMPL_ENGINE_SYNC_ASYNC_IMPL_HPP

#include <impl/config.hpp>

#include <boost/preprocessor/iterate.hpp>
#include <boost/preprocessor/repetition/enum_params.hpp>
#include <boost/preprocessor/repetition/enum_binary_params.hpp>

#define BOOST_PP_ITERATION_PARAMS_1                                           \
    (3, (1, SAGA_ARGUMENT_LIMIT, "impl/engine/sync_async_impl.hpp"))          \
    /**/

#include BOOST_PP_ITERATE()

#endif // SAGA_IMPL_ENGINE_SYNC_ASYNC_IMPL_HPP

///////////////////////////////////////////////////////////////////////////////
//
//  Preprocessor vertical repetition code
//
///////////////////////////////////////////////////////////////////////////////
#else // defined(BOOST_PP_IS_ITERATING)

#define K BOOST_PP_ITERATION()

    ///////////////////////////////////////////////////////////////////////////
    // need sync, have sync
    template <typename Cpi, typename FuncBase, typename FuncRetVal, 
              typename RetVal, 
              BOOST_PP_ENUM_PARAMS (K, typename FuncArg),
              BOOST_PP_ENUM_PARAMS (K, typename Arg)>
    inline task_type 
    sync_sync (TR1::shared_ptr <Cpi> cpi_instance,
               void (FuncBase::*sync_func) (FuncRetVal, 
                                           BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
               RetVal ret, 
               BOOST_PP_ENUM_BINARY_PARAMS (K, Arg, const& arg))
    {
        TR1::bind (sync_func, cpi_instance, ret, BOOST_PP_ENUM_PARAMS (K, arg)) ();
        
        return (task_type (task_type::Done)); 
    }

    
    // need async, have sync
    template <typename Cpi, typename FuncBase, typename FuncRetVal, 
              typename RetVal, 
              BOOST_PP_ENUM_PARAMS (K, typename FuncArg),
              BOOST_PP_ENUM_PARAMS (K, typename Arg)>
    inline task_type 
    async_sync (TR1::shared_ptr<Cpi> cpi_instance,
                void (FuncBase::*sync_func) (FuncRetVal, 
                                            BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
                RetVal ret,
                char const *func_name_,
                BOOST_PP_ENUM_BINARY_PARAMS (K, Arg, const& arg),
                bool (FuncBase::*prep_func_)(FuncRetVal,
                                            BOOST_PP_ENUM_PARAMS (K, FuncArg), 
                                            saga::uuid) = NULL)
    {
        return (saga::adaptors::task(func_name_,
                                    cpi_instance,/*shp*/
                                    sync_func, 
                                    ret,
                                    BOOST_PP_ENUM_PARAMS (K, arg),
                                    prep_func_ ) ); 
    }
    

    ///////////////////////////////////////////////////////////////////////////
    // need sync, have async
    template <typename Cpi, typename FuncBase, typename FuncRetV, 
              typename RetVal, 
              BOOST_PP_ENUM_PARAMS (K, typename FuncArg),
              BOOST_PP_ENUM_PARAMS (K, typename Arg)>
    inline task_type 
    sync_async (TR1::shared_ptr <Cpi> cpi_instance,
                task_type (FuncBase::*async_func) (FuncRetV, 
                                                   BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
                RetVal ret, 
                BOOST_PP_ENUM_BINARY_PARAMS (K, Arg, const& arg))
    {
        return saga::detail::run_wait (TR1::bind (async_func, cpi_instance, ret, 
                            BOOST_PP_ENUM_PARAMS (K, arg)) ());
    }

    ///////////////////////////////////////////////////////////////////////////
    // need async, have async
    template <typename Cpi, typename FuncBase, typename FuncRetV, 
              typename RetVal, 
              BOOST_PP_ENUM_PARAMS (K, typename FuncArg),
              BOOST_PP_ENUM_PARAMS (K, typename Arg)>
    inline task_type 
    async_async (TR1::shared_ptr <Cpi> cpi_instance,
                 task_type (FuncBase::*async_func) (FuncRetV, 
                                                    BOOST_PP_ENUM_PARAMS(K, FuncArg)), 
                 RetVal ret, 
                 BOOST_PP_ENUM_BINARY_PARAMS (K, Arg, const& arg))
    {
        return TR1::bind (async_func, cpi_instance, ret, 
                            BOOST_PP_ENUM_PARAMS (K, arg)) ();
    }

   
    ///////////////////////////////////////////////////////////////////////////
    template <typename FuncBase, typename FuncRetVal, typename RetVal, 
              BOOST_PP_ENUM_PARAMS (K, typename FuncArg),
              BOOST_PP_ENUM_PARAMS (K, typename Arg)>
    inline task_type 
    dispatch_sync_async (run_mode                           mode,
                         TR1::shared_ptr <v1_0::cpi>      cpi_instance, 
                         void (FuncBase::*sync_func)        (FuncRetVal, 
                                        BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
                         task_type (FuncBase::*async_func) (FuncRetVal, 
                                        BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
                         char const                       * name, 
                         RetVal                             ret, 
                         BOOST_PP_ENUM_BINARY_PARAMS (K, Arg, const& arg),
                         bool (FuncBase::*prep_func_)(FuncRetVal,
                                        BOOST_PP_ENUM_PARAMS(K, FuncArg),
                                        saga::uuid) = NULL)
    {    
        TR1::shared_ptr <FuncBase> c =
            TR1::static_pointer_cast<FuncBase>(cpi_instance);
                   
        switch ( mode ) 
        {
          case Sync_Sync:
               return sync_sync   (c, sync_func, ret,
                                   BOOST_PP_ENUM_PARAMS(K, arg));
          case ASync_Sync:
               return async_sync  (c, sync_func, ret, name,
                                   BOOST_PP_ENUM_PARAMS(K, arg), prep_func_);
          case Sync_ASync:
               return sync_async  (c, async_func, ret, 
                                   BOOST_PP_ENUM_PARAMS(K, arg));
          case ASync_ASync:
               return async_async (c, async_func, ret, 
                                   BOOST_PP_ENUM_PARAMS(K, arg));
          default:
               break;
        }

        // no adaptor found!
        SAGA_THROW_VERBATIM(cpi_instance.get(),
            std::string ("No adaptor implements method: ") + name, NoAdaptor);
        return task_type(task_type::Done);
    }

    ///////////////////////////////////////////////////////////////////////////
    template <
        typename FuncBase, typename FuncRetVal, typename RetVal,
        BOOST_PP_ENUM_PARAMS (K, typename FuncArg),
        BOOST_PP_ENUM_PARAMS (K, typename Arg)
    >
    inline task_type 
    execute_sync_async(proxy *prxy,
        char const *cpi_name, char const *name, char const *op_name, 
        v1_0::preference_type const& prefs, bool is_sync,
        void (FuncBase::*sync_func)(FuncRetVal, BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
        task_type (FuncBase::*async_func)(FuncRetVal, BOOST_PP_ENUM_PARAMS (K, FuncArg)), 
        RetVal ret, BOOST_PP_ENUM_BINARY_PARAMS (K, Arg, const& arg));

#undef K

#endif // defined(BOOST_PP_IS_ITERATING)

