//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_MONITORABLE_HPP
#define SAGA_MONITORABLE_HPP

// include STL
#include <string>
#include <vector>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable : 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    /// @cond
    /** These methods are not within API scope */
    typedef bool callback_function_type(saga::object, saga::metric, saga::context);
    typedef TR1::function<callback_function_type> callback;
    /// @endcond
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_EXPORT monitorable
    {
    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::object> impl_;

        saga::impl::monitorable* get_monitorable (void);
        saga::impl::monitorable const* get_monitorable (void) const;
        /// @endcond

    public:
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        typedef unsigned int cookie_handle;
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        monitorable (saga::object rhs);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        explicit monitorable (saga::impl::object *impl);    // adaptor interface
    
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual ~monitorable (void);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual std::vector<saga::metric> list_metrics (void) const;
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual saga::metric get_metric (std::string name) const;

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual cookie_handle add_callback (std::string name, saga::callback cb);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual void remove_callback (std::string name, cookie_handle  cookie);
    };

    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_EXPORT steerable
    :   public monitorable
    {
    protected:
        /// @cond
        /** These methods are not within API scope */
        saga::impl::steerable* get_steerable (void);
        saga::impl::steerable const* get_steerable (void) const;
        /// @endcond

    public:
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        steerable (saga::object rhs);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        explicit steerable (saga::impl::object *impl);    // adaptor interface

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual ~steerable (void);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual bool add_metric (saga::metric m);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual void remove_metric (std::string name);
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        virtual void fire_metric (std::string name);
    };

///////////////////////////////////////////////////////////////////////////////
} // namespace saga
                                                                             
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_MONITORABLE_HPP

