//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_SAGA_OBJECT_HPP
#define SAGA_SAGA_OBJECT_HPP

#include <saga/util.hpp>
#include <saga/base.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
//
//  SAGA spec:
//
//  package saga.object
//  {
//    enum object_type 
//    {
//      Unknown          =  -1,
//      exception        =   1,
//      Session          =   2,
//      Context          =   3,
//      Task             =   4,
//      TaskContainer    =   5,
//      Metric           =   6,
//      NSEntry          =   7,
//      NSDirectory      =   8,
//      File             =   9,
//      Directory        =  10,
//      LogicalFile      =  11,
//      LogicalDirectory =  12,
//      JobDescription   =  13,
//      JobService       =  14,
//      Job              =  15,
//      StreamServer     =  16,
//      Stream           =  17,
//      Multiplexer      =  18
//    }
// 
//    interface object 
//    {  
//      get_id       (out string      id     );
//      get_type     (out object_type type   );
//      get_session  (out session     session);
//    }
//  }
///////////////////////////////////////////////////////////////////////////////
namespace saga
{
  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  class SAGA_EXPORT object 
  {
    public:
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      enum type 
      {
          // FIXME: this conflicts with the Unknown context type
          // Unknown          =  -1,
          Exception        =   1,
          Session          =   2,
          Context          =   3,
          Task             =   4,
          TaskContainer    =   5,
          Buffer           =   6,
          Metric           =   7,
          NSEntry          =   8,
          NSDirectory      =   9,
          IOVec            =  10,
          File             =  11,
          Directory        =  12,
          LogicalFile      =  13,
          LogicalDirectory =  14,
          JobDescription   =  15,
          JobService       =  16,
          Job              =  17,
          JobSelf          =  18,
          StreamService    =  19,
          Stream           =  20,
          RPC              =  21,
          Parameter        =  22,
        
      // SAGA Message API extension
          Msg              =  23,
          Endpoint         =  24,
        
      // SAGA Information services extension
          Advert           =  25,
          AdvertDirectory  =  26
      };

    private:
      /// @cond
      /** These methods are not within API scope */
      TR1::shared_ptr <saga::impl::object> impl_;

      friend struct impl::runtime;
      friend class  attribute;
      friend class  adaptors::attribute;
      friend class  monitorable;
      friend class  permissions;
      /// @endcond
            
    protected:
      /// @cond
      /** These methods are not within API scope */
      TR1::shared_ptr <saga::impl::object> get_impl (void) const
      { 
        return impl_;
      }

      explicit object (saga::impl::object  * init);
      explicit object (TR1::shared_ptr<saga::impl::object> init);
      /// @endcond
     
    public:
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      object();

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      saga::object::type  get_type     (void) const;
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      saga::uuid          get_id       (void) const;

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      saga::session       get_session  (void) const;
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      saga::object clone() const;

  }; // object

  /// @cond
  /** These methods are not within API scope TODO wrong namespace?!? */
  SAGA_EXPORT std::string get_object_type_name(saga::object::type t);
  /// @endcond
  
} // namespace saga
///////////////////////////////////////////////////////////////////////////////

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#  pragma warning(pop)
#endif

#endif // SAGA_SAGA_OBJECT_HPP

