#ifndef SAGA_SAGA_PACKAGES_DATA_LOGICAL_FILE_ADVERT_DIRECTORY_HPP
#define SAGA_SAGA_PACKAGES_DATA_LOGICAL_FILE_ADVERT_DIRECTORY_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/directory.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

// include stl
#include <string>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/call.hpp>
#include <saga/types.hpp>
#include <saga/object.hpp>
#include <saga/session.hpp>
#include <saga/detail/attribute.hpp>
#include <saga/detail/monitorable.hpp>

#include <saga/packages/namespace/namespace_dir.hpp>
#include <saga/packages/advert/advert.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/directory.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/directory.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace advert { 
        
        namespace metrics 
        {
            /*! \brief Brief description here 
             *
             *   Detailed %description here...
             */ 
            char const* const directory_created_entry  = "directory.CreatedEntry";
            
            /*! \brief Brief description here 
             *
             *   Detailed %description here...
             */ 
            char const* const directory_modified_entry = "directory.ModifiedEntry";
            
            /*! \brief Brief description here 
             *
             *   Detailed %description here...
             */ 
            char const* const directory_deleted_entry  = "directory.DeletedEntry";
        }
        
        /*! \brief The %directory class represents a <b>directory</b> within  
         *         a meta data %directory with a hierarchical namespace attached.  
         *         This class inherits all methods from saga::name_space::directoryectory.      
         *
         * Detailed %description starts here.
         */
        class SAGA_ADVERT_PACKAGE_EXPORT directory 
        : public saga::name_space::directory, public saga::detail::attribute<directory>
        {
        protected:
            /// @cond
            /** These types are not within API scope */
            friend class saga::impl::advert_directory;
            
            friend struct saga::detail::attribute<directory>;  
            typedef saga::detail::attribute<directory> attribute_base;
            typedef saga::name_space::directory::monitorable_base monitorable_base;
            /// @endcond
            
        public:                 
            
        private:
            // overload open methods from namespace_dir
            SAGA_CALL_PRIV_2 (advert::entry, open, std::string, int)
            SAGA_CALL_PRIV_2 (advert::directory, open_dir, std::string, int)
            SAGA_CALL_PRIV_3 (std::vector<std::string>, find, std::string, std::vector<std::string>, int)
            
        protected:
            /// @cond
            /** These methods are not within API scope */
            TR1::shared_ptr <saga::impl::advert_directory> get_impl() const;
            explicit directory(saga::impl::advert_directory *impl);
            /// @endcond        
            
        public:
            /*! \brief Constructs an %directory %object that points to %url,  
             *         opens it in mode and associates it with the %session s.
             */   
            directory (saga::session const& s, std::string const& url, 
                       int mode = saga::advert::ReadWrite);
            
            /*! \brief Constructs an %directory %object that points to %url  
             *         and opens it in mode. 
             */                     
            explicit directory (std::string const & url, 
                                int mode = saga::advert::ReadWrite);
            
            /*! \brief Constructs a copy of other.
             */                            
            explicit directory (saga::object const& other);
            
            /*! \brief Tries to onstructs a default %directory %object.
             */        
            directory (void);
            
            /*! \brief Destroys the %directory %object and frees all  
             *         allocated resources.
             */        
            ~directory (void);
            
            /*! \brief Assigns %object to this %directory and returns 
             *         a reference to it. <b>TASK ONLY</b>
             */        
            directory &operator= (saga::object const& o);                                                      
            
            /*! \brief Tries to open the %advert at the URL url with mode.
             */   
            entry open (std::string url, int mode = saga::advert::Read) { 
                saga::advert::entry ret; 
                openpriv (ret, url, mode, saga::task_base::Sync()); 
                return ret;
            }
            SAGA_CALL_PUB_2_DEF_1 (entry, open, std::string, int, saga::advert::Read)
            
            /*! \brief Tries to open the %advert_direcory at URL url with mode.
             *
             */   
            directory open_dir (std::string url, 
                                int mode = saga::advert::None) { 
                saga::advert::directory ret; 
                open_dirpriv (ret, url, mode, saga::task_base::Sync()); 
                return ret;
            }
            SAGA_CALL_PUB_2_DEF_1 (directory, open_dir, std::string, int, saga::advert::None)
            
            /*! \brief Tries to find %directory entries which match...
             *
             */
            std::vector<std::string> find (std::string name, 
                                           std::vector<std::string> pattern, 
                                           int flags = saga::advert::None)
            { 
                std::vector<std::string> ret; 
                findpriv (ret, name, pattern, flags, saga::task_base::Sync()); 
                return ret; 
            }
            SAGA_CALL_PUB_3_DEF_1 (std::vector<std::string>, find, std::string, std::vector<std::string>, int, saga::advert::None)
        }; // class directory
    }; // namespace 
}; // namespace saga

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_SAGA_PACKAGES_DATA_LOGICAL_FILE_ADVERT_DIRECTORY_HPP
