//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_PACKAGE_DATA_ADVERT_VERSION_HPP
#define SAGA_PACKAGE_DATA_ADVERT_VERSION_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

#include <saga/util.hpp>
#include <saga/packages/advert/config.hpp>

///////////////////////////////////////////////////////////////////////////////
//  The version of the SAGA file package
//
//  SAGA_VERSION_PACKAGE_ADVERT_FULL & 0x0000FF is the sub-minor version
//  SAGA_VERSION_PACKAGE_ADVERT_FULL & 0x00FF00 is the minor version
//  SAGA_VERSION_PACKAGE_ADVERT_FULL & 0xFF0000 is the major version
//
#define SAGA_VERSION_PACKAGE_ADVERT_FULL         0x000500

#define SAGA_VERSION_PACKAGE_ADVERT_MAJOR        0
#define SAGA_VERSION_PACKAGE_ADVERT_MINOR        5
#define SAGA_VERSION_PACKAGE_ADVERT_SUBMINOR     0

#define SAGA_VERSION_PACKAGE_ADVERT_DATE         070323

#define SAGA_VERSION_PACKAGE_ADVERT_SUBMINORMASK 0xFF

///////////////////////////////////////////////////////////////////////////////
//  The SAGA file package API version: here V1.0.0
#define SAGA_VERSION_PACKAGE_ADVERT_API          0x010000

///////////////////////////////////////////////////////////////////////////////
//
//  SAGA_VERSION_PACKAGE_ADVERT_ISCOMPATIBLE()
// 
//  The SAGA_VERSION_ISCOMPATIBLE macro tests, if the version of the loaded
//  SAGA engine is compatible with the version the calling application was 
//  compiled against.
//
#define SAGA_VERSION_PACKAGE_ADVERT_ISCOMPATIBLE_EX(version)                  \
    (((version) & ~SAGA_VERSION_PACKAGE_ADVERT_SUBMINORMASK) >=               \
      (SAGA_VERSION_PACKAGE_ADVERT_FULL &                                     \
          ~SAGA_VERSION_PACKAGE_ADVERT_SUBMINORMASK))                         \
  /**/

#define SAGA_VERSION_PACKAGE_ADVERT_ISCOMPATIBLE()                            \
    SAGA_VERSION_PACKAGE_ADVERT_ISCOMPATIBLE_EX(                              \
        saga::get_advert_package_version())                                   \
  /**/
  
///////////////////////////////////////////////////////////////////////////////
namespace saga {

  // Returns the version of the SAGA engine
  SAGA_ADVERT_PACKAGE_EXPORT 
  unsigned long get_advert_package_version (void);

  // Returns the implemented SAGA API version
  SAGA_ADVERT_PACKAGE_EXPORT 
  unsigned long get_advert_package_api_version (void);

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif // SAGA_PACKAGE_DATA_ADVERT_VERSION_HPP


