#ifndef SAGA_PACKAGES_JOB_JOB_SERVICE_HPP
#define SAGA_PACKAGES_JOB_JOB_SERVICE_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/job_service.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#include <string>
#include <vector>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/types.hpp>
#include <saga/session.hpp>
#include <saga/call.hpp>

#include <saga/packages/job/config.hpp>
#include <saga/job.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/job/preprocessed/job_service.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/job_service.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    namespace job {
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_JOB_PACKAGE_EXPORT service 
        : public saga::object
    {

    private:
        SAGA_CALL_PRIV_1 (job, create_job, description);
        SAGA_CALL_PRIV_5 (job, run_job, std::string, std::string,
            saga::job::ostream&, saga::job::istream&, saga::job::istream&);
        SAGA_CALL_PRIV_0 (std::vector<std::string>, list);
        SAGA_CALL_PRIV_1 (job, get_job, std::string);
        SAGA_CALL_PRIV_0 (self, get_self);

    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::service> get_impl (void) const;
        /// @endcond

    public:
        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        explicit service (session const& s, std::string const& rm = "");

        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        explicit service (std::string const& rm = "");

        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */     
        explicit service (saga::object const& o);

        /*! \brief Destroys the object.     
         *
         *   Detailed %description starts here.
         */
        ~service (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        service& operator= (saga::object const& o);

        /*! \brief Create a job instance     
         *  \exception saga::exception (on condition)
         *  \param job_desc Description of job to be submitted 
         *  \return A job object representing the submitted job instance 
         *
         *   Detailed %description starts here.
         */
        job create_job(description job_desc) {
            job ret;
            create_jobpriv (ret, job_desc, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (job, create_job, description);

        /*! \brief Run a command synchronously    
         *  \exception saga::exception (on condition)
         *  \param hostname Hostname to be used by rm for submission 
         *  \param commandline The command and arguments to be run 
         *
         *   Detailed %description starts here.
         */
        job run_job(std::string hostname, std::string commandline,
                    saga::job::ostream& stdin_stream, 
                    saga::job::istream& stdout_stream, saga::job::istream& stderr_stream) 
        {
            job ret;
            run_jobpriv (ret, hostname, commandline, stdin_stream, stdout_stream, 
                      stderr_stream, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_5_DEF_0 (job, run_job, std::string, std::string,
            saga::job::ostream&, saga::job::istream&, saga::job::istream&);

        /*! \brief Get a list of jobs which are currently known by 
         *         the resource manager    
         *  \return An array of job identifiers 
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        std::vector<std::string> list(void) {
            std::vector<std::string> ret;
            listpriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_0_DEF_0 (std::vector<std::string>, list);

        /*! \brief Given a job identifier, this method returns a job object 
         *         representing this job    
         *  \exception saga::exception (on condition)
         *  \param job_id Job identifier as returned by the resource manager 
         *  \return A job object representing the job identified by job_id
         *
         *   Detailed %description starts here.
         */
        job get_job(std::string job_id) {
            self ret;
            get_jobpriv (ret, job_id, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (job, get_job, std::string);

        /*! \brief This method returns a job object representing _this_ job, 
         *         i.e. the calling application 
         *  \exception saga::exception (on condition)
         *  \return A job_self object representing _this_ job.
         *
         *   Detailed %description starts here.
         */
        self get_self(void) {
            self ret;
            get_selfpriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_0_DEF_0 (self, get_self);

    };
     
    }   // namespace job
}  

///////////////////////////////////////////////////////////////////////////////
#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // !defined(SAGA_PACKAGES_JOB_JOB_SERVICE_HPP)

