//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_PACKAGE_JOB_VERSION_HPP
#define SAGA_PACKAGE_JOB_VERSION_HPP

#include <saga/util.hpp>
#include <saga/packages/job/config.hpp>

///////////////////////////////////////////////////////////////////////////////
//  The version of the SAGA file package
//
//  SAGA_VERSION_PACKAGE_JOB_FULL & 0x0000FF is the sub-minor version
//  SAGA_VERSION_PACKAGE_JOB_FULL & 0x00FF00 is the minor version
//  SAGA_VERSION_PACKAGE_JOB_FULL & 0xFF0000 is the major version
//
#define SAGA_VERSION_PACKAGE_JOB_FULL         0x000500

#define SAGA_VERSION_PACKAGE_JOB_MAJOR        0
#define SAGA_VERSION_PACKAGE_JOB_MINOR        5
#define SAGA_VERSION_PACKAGE_JOB_SUBMINOR     0

#define SAGA_VERSION_PACKAGE_JOB_DATE         070323

#define SAGA_VERSION_PACKAGE_JOB_SUBMINORMASK 0xFF

///////////////////////////////////////////////////////////////////////////////
//  The SAGA file package API version: here V1.0.0
#define SAGA_VERSION_PACKAGE_JOB_API          0x010000

///////////////////////////////////////////////////////////////////////////////
//
//  SAGA_VERSION_PACKAGE_JOB_ISCOMPATIBLE()
// 
//  The SAGA_VERSION_ISCOMPATIBLE macro tests, if the version of the loaded
//  SAGA engine is compatible with the version the calling application was 
//  compiled against.
//
#define SAGA_VERSION_PACKAGE_JOB_ISCOMPATIBLE_EX(version)                     \
    (((version) & ~SAGA_VERSION_PACKAGE_JOB_SUBMINORMASK) >=                  \
      (SAGA_VERSION_PACKAGE_JOB_FULL & ~SAGA_VERSION_PACKAGE_JOB_SUBMINORMASK)) \
  /**/

#define SAGA_VERSION_PACKAGE_JOB_ISCOMPATIBLE()                               \
    SAGA_VERSION_PACKAGE_JOB_ISCOMPATIBLE_EX(saga::get_job_package_version()) \
  /**/

///////////////////////////////////////////////////////////////////////////////
namespace saga {

  // Returns the version of the SAGA jobpackage
  SAGA_JOB_PACKAGE_EXPORT unsigned long get_job_package_version (void);

  // Returns the implemented SAGA job package API version
  SAGA_JOB_PACKAGE_EXPORT unsigned long get_job_package_api_version (void);

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif // SAGA_PACKAGE_JOB_VERSION_HPP


