#ifndef SAGA_SAGA_PACKAGES_DATA_LOGICAL_FILE_LOGICAL_DIRECTORY_HPP
#define SAGA_SAGA_PACKAGES_DATA_LOGICAL_FILE_LOGICAL_DIRECTORY_HPP

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_directory.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

// include stl
#include <string>

// include dependend spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/call.hpp>
#include <saga/types.hpp>
#include <saga/object.hpp>
#include <saga/session.hpp>
#include <saga/detail/attribute.hpp>

#include <saga/packages/namespace/namespace_dir.hpp>
#include <saga/packages/logicalfile/logical_file.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/logical_directory.hpp>
#else

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_directory.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  /////////////////////////////////////////////////////////////////////////////
  namespace metrics 
  {
    ///////////////////////////////////////////////////////////////////////////
    //  metric names for a advert entries
    char const* const logical_directory_created_entry = "logical_directory.CreatedEntry";
    char const* const logical_directory_modified_entry = "logical_directory.ModifiedEntry";
    char const* const logical_directory_deleted_entry = "logical_directory.DeletedEntry";
  }


    namespace logical_file {

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
    class SAGA_LOGICAL_FILE_PACKAGE_EXPORT logical_directory 
      : public saga::name_space::directory,
        public saga::detail::attribute<logical_directory>
    {
    protected:
        /// @cond
        /** These methods are not within API scope */
        friend class saga::impl::logical_directory;
        friend struct saga::detail::attribute<logical_directory>;  // needs to access get_impl()
        typedef saga::detail::attribute<logical_directory> attribute_base;
        typedef saga::name_space::directory::monitorable_base monitorable_base;
        /// @endcond                  

    private:
        /// @cond
        /** These methods are not within API scope */
        SAGA_CALL_PRIV_1 (bool, is_file, std::string)

        // overload open methods from namespace_dir
        SAGA_CALL_PRIV_2 (saga::logical_file::logical_file, open, std::string, int)
        SAGA_CALL_PRIV_2 (saga::logical_file::logical_directory, open_dir, std::string, int)
        SAGA_CALL_PRIV_3 (std::vector<std::string>, find, std::string, std::vector<std::string>, int)
        /// @endcond
    
    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::logical_directory> get_impl() const;

        explicit logical_directory(saga::impl::logical_directory *impl);
        /// @endcond

    public:
        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        logical_directory (session const& s, std::string const& url, 
                           int mode = saga::logical_file::ReadWrite);

        /*! \brief Creates the object.       
         *
         *   Detailed %description starts here.
         */
        explicit logical_directory (std::string const & url, 
                                    int mode = saga::logical_file::ReadWrite);

        /*! \brief Creates the object.       
         *
         *   Detailed %description starts here.
         */
        explicit logical_directory (saga::object const& o);

        /*! \brief Creates the object.       
         *
         *   Detailed %description starts here.
         */
        logical_directory (void);

        /*! \brief Destroys the object.     
         *
         *   Detailed %description starts here.
         */
        ~logical_directory (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        logical_directory& operator=(saga::object const& o);

        /*! \brief Alias for is_entry of saga::ns_directory     
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        bool is_file(std::string url) {
            bool ret;
            is_filepriv (ret, url, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (bool, is_file,  std::string)

        /*! \brief Creates a new logical_file instance.
         *  \exception saga::exception (on condition)
         *  \param url File to be opened 
         *  \param flags Flags definition operation modus 
         *  \return Opened file instance
         *
         *   Detailed %description starts here.
         */
        saga::logical_file::logical_file open(std::string url, int flags=saga::logical_file::Read) {
            saga::logical_file::logical_file ret;
            openpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::logical_file::logical_file, open, std::string, int, saga::logical_file::Read)

        /*! \brief Creates a new logical_directory instance     
         *  \exception saga::exception (on condition)
         *  \param url Name of directory to open 
         *  \param flags Flags definition operation modus 
         *  \return Opened directory instance 
         *
         *   Detailed %description starts here.
         */
        saga::logical_file::logical_directory open_dir(std::string url, int flags=saga::logical_file::None) {
                saga::logical_file::logical_directory ret;
            open_dirpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::logical_file::logical_directory, open_dir, std::string, int, saga::logical_file::None)

        /*! \brief Find entries in the current directory and below, with matching
         *         names and matching meta data      
         *  \exception saga::exception (on condition)
         *  \param name_pattern Pattern for names of entries to be found 
         *  \param key_pattern Pattern for meta data of entries to be found 
         *  \param flags Flags defining the operation modus 
         *  \return Array of names matching both pattern 
         *
         *   Detailed %description starts here.
        */
        std::vector<std::string> find(std::string name_pattern, 
                                      std::vector<std::string> key_pattern,
                                      int flags=saga::logical_file::Recursive) 
        {
            std::vector<std::string> ret;
            findpriv (ret, name_pattern, key_pattern, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_3_DEF_1 (std::vector<std::string>, find, std::string,  std::vector<std::string>, int, saga::logical_file::None)
    }; 

    }
}

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_SAGA_PACKAGES_DATA_LOGICAL_FILE_LOGICAL_DIRECTORY_HPP
