//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_PACKAGES_COMM_RPC_RPC_HPP
#define SAGA_PACKAGES_COMM_RPC_RPC_HPP

// include stl
#include <string>
#include <vector>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/types.hpp>
#include <saga/session.hpp>
#include <saga/call.hpp>
#include <saga/task.hpp>

#include <saga/detail/permissions.hpp>

#include <saga/packages/rpc/config.hpp>
#include <saga/packages/rpc/parameter.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

namespace saga
{
    /*! \brief The <I>Remote Procedure Call </I> API package
     *
     *   Detailed %description here...
     */   
    namespace rpc {
 
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */         
     class SAGA_RPC_PACKAGE_EXPORT rpc 
        : public saga::object,
          public saga::detail::permissions<rpc>
    {

    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::rpc> get_impl (void) const;
        friend struct saga::detail::permissions<rpc>;
        /// @endcond
      
    private:
        SAGA_CALL_PRIV_1 (void_t, call, std::vector<parameter>&);
        SAGA_CALL_PRIV_1 (void_t, close, double);

    public:
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */       
        rpc (session const& s, std::string const& name = "");

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */      
        explicit rpc (std::string const& name);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */       
        rpc (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */      
        ~rpc (void);

        /*! \brief Call the remote procedure 
         *
         *   Detailed %description starts here.
         */ 
        saga::void_t call(std::vector<parameter> parameters) {
            saga::void_t ret;
            callpriv (ret, parameters, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::void_t, call, std::vector<parameter>&);

        /*! \brief Closes the rpc handle instance     
         *
         *   Detailed %description starts here.
         */  
        saga::void_t close(double timeout=0.0) {
            saga::void_t ret;
            closepriv (ret, timeout, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_1 (saga::void_t, close, double, 0.0);
    }; 
    };
}

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_PACKAGES_COMM_RPC_RPC_HPP

