//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_PERMISSIONS_HPP
#define SAGA_PERMISSIONS_HPP

// include STL
#include <string>
#include <vector>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>

#include <saga/detail/permissions.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable : 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    enum permission
    {
        None = 0,
        Query = 1,
        Read = 2,
        Write = 4,
        Exec = 8,
        Owner = 0x10,
        All = 0x1F
    };
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_EXPORT permissions
      : public detail::permissions<permissions>
    {
    protected:
        /// @cond
        /** These methods are not within API scope */
        friend struct detail::permissions<permissions>;  // needs to access get_impl()
        TR1::shared_ptr <saga::impl::object> impl_;

        TR1::shared_ptr <saga::impl::object> get_impl (void) const
            { return impl_; }
        /// @endcond

    public:
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        permissions (saga::object rhs);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        explicit permissions (saga::impl::object *impl);    // adaptor interface

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */        
        ~permissions (void);
    };

///////////////////////////////////////////////////////////////////////////////
} // namespace saga
                                                                             
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_PERMISSIONS_HPP

