//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_SESSION_HPP
#define SAGA_SESSION_HPP

// include stl
#include <list>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/object.hpp>
#include <saga/context_base.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    ///@cond
    namespace detail {
        // the default session as singleton, and the no-session singleton
        SAGA_EXPORT session        get_the_session ();
        #define SAGA_DEFAULT_SESSION detail::get_the_session ()
    }
    ///@endcond

  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  class SAGA_EXPORT session
    : public saga::object
  {
    private: 
      /// @cond 
      /** These methods are not within API scope */
      TR1::shared_ptr <saga::impl::session> get_impl (void) const;

      friend class  saga::object; // needs to get impl
      friend struct impl::runtime;
      /// @endcond

    public:
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      session (void);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      session (saga::object const& obj);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
     ~session (void);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      session &operator= (saga::object const& o);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      void add_context    (saga::context const & c);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      void remove_context (saga::context const & c);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      std::list <saga::context> 
      list_contexts (saga::context_base::type t = saga::context_base::Any) const;
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      friend SAGA_EXPORT 
        bool operator== (session const & lhs, session const & rhs);
  };

///////////////////////////////////////////////////////////////////////////////
} // namespace saga

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_SESSION_HPP

