//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2006 Stephan Hirmer (shirmer@cct.lsu.edu)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_TASK_V1_HPP
#define SAGA_TASK_V1_HPP

// include dependent spec sections
#include <saga/base.hpp>
#include <saga/util.hpp>
#include <saga/task_base.hpp>
#include <saga/object.hpp>

#include <saga/uuid.hpp>
#include <saga/detail/monitorable.hpp>

// #include <impl/engine/task_interface.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  ///////////////////////////////////////////////////////////////////////////////
  namespace metrics 
  {
    ///////////////////////////////////////////////////////////////////////////
    //  metric names for task
    char const* const task_state = "task.State";
  }
  
#if defined(SAGA_USE_V2_TASK_MODEL)
  namespace detail 
  {
#endif

  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  class SAGA_EXPORT task 
    : public object,
      public detail::monitorable<task>,
      public task_base
  {
    private:
      /// @cond
      /** These methods are not within API scope */
      friend struct saga::impl::runtime;
      friend struct detail::monitorable<task>;  // needs to access get_impl()
      /// @endcond
      
    protected:
      /// @cond
      /** These methods are not within API scope */
      saga::impl::task_interface* get_task_if (void);
      saga::impl::task_interface const* get_task_if (void) const;
      typedef detail::monitorable<task> monitorable_base;

      explicit task (saga::impl::object* init);
      explicit task (saga::object const& o);
      task &operator= (saga::object const& o);
      /// @endcond

    public:
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      typedef task_base::state state;

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      explicit task (saga::task_base::state t = saga::task_base::New);
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      ~task ();

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      friend SAGA_EXPORT 
        bool operator== (task const & lhs, task const & rhs);

      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      void  run       (void);
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      void  cancel    (void);
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      bool  wait      (double timeout = -1.0);
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      state get_state (void) const;
      
      /*! \brief Brief %description starts here     
       *
       *   Detailed %description starts here.
       */
      void rethrow() const;
      
      saga::object get_object() const;
  };

#if defined(SAGA_USE_V2_TASK_MODEL)
  }
#endif  // !SAGA_USE_V2_API
  namespace detail {
      /////////////////////////////////////////////////////////////////////////////
      // some global functions to simplify the implementation of sync/async/task
      // versions of method calls

      /// @cond
      /** These methods are not within API scope TODO: wrong namespace?!?*/
      SAGA_EXPORT inline task_type run (task_type t) 
      { 
        t.run (); 
        return t; 
      }

      SAGA_EXPORT inline task_type wait (task_type t, double timeout = -1.0) 
      { 
        t.wait (timeout); 
        return t; 
      }

      SAGA_EXPORT inline task_type run_wait (task_type t, double timeout = -1.0) 
      { 
        if ( saga::task_base::New == t.get_state () ) 
        {
          t.run  (); 
          t.wait (timeout); 
        }
        return t; 
      }
   }  
} // namespace saga
///////////////////////////////////////////////////////////////////////////////

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_TASK_V1_HPP

