//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_VERSION_HPP
#define SAGA_VERSION_HPP

#include <saga/util.hpp>

///////////////////////////////////////////////////////////////////////////////
//  The version of the SAGA engine
//
//  SAGA_VERSION_FULL & 0x0000FF is the sub-minor version
//  SAGA_VERSION_FULL & 0x00FF00 is the minor version
//  SAGA_VERSION_FULL & 0xFF0000 is the major version
//
#define SAGA_VERSION_FULL         0x000800

#define SAGA_VERSION_MAJOR        0
#define SAGA_VERSION_MINOR        8
#define SAGA_VERSION_SUBMINOR     0

#define SAGA_VERSION_DATE         070920

#define SAGA_VERSION_SUBMINORMASK 0xFF

///////////////////////////////////////////////////////////////////////////////
//  The SAGA API version: here V1.0.0
#define SAGA_VERSION_API          0x010000

///////////////////////////////////////////////////////////////////////////////
//
//  SAGA_VERSION_ISCOMPATIBLE(), SAGA_VERSION_ISCOMPATIBLE_EX()
// 
//  The SAGA_VERSION_ISCOMPATIBLE macros test, if the version of the loaded
//  SAGA engine is compatible with the version the calling application was 
//  compiled against.
//
#define SAGA_VERSION_ISCOMPATIBLE_EX(version)                                 \
    (((version) & ~SAGA_VERSION_SUBMINORMASK) >=                              \
       (SAGA_VERSION_FULL & ~SAGA_VERSION_SUBMINORMASK))                      \
  /**/
  
#define SAGA_VERSION_ISCOMPATIBLE()                                           \
    SAGA_VERSION_ISCOMPATIBLE_EX(saga::get_engine_version())                  \
  /**/

/*! \brief The SAGA root namespace
 */
namespace saga 
{
  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  SAGA_EXPORT unsigned long get_engine_version();
  
  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  SAGA_EXPORT unsigned long get_saga_version();
  
} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif // !SAGA_VERSION_HPP

