//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>

#include <saga.hpp>

///////////////////////////////////////////////////////////////////////////////
void usage (int val = -1)
{
  std::cout << "Usage: job_run <hostname> <exe> [<args>]..." 
            << std::endl;
  exit (val);
}

///////////////////////////////////////////////////////////////////////////////
int main (int argc, char* argv[])
{
  if ( argc < 3 ) 
  {
    usage ();
  }

  std::string host   = "any://"+std::string(argv[1]);
  std::string exe     (argv[2]);
  std::vector<std::string> args;

  for ( int i = 3; i < argc; i++ )
  {
    args.push_back(argv[i]);
  }

  try 
  {
    saga::job::job_description jd;
    saga::job::job_service     js (host);

  // create io streams for job io
    saga::job::ostream in;
    saga::job::istream out;
    saga::job::istream err;

    jd.set_attribute (saga::attributes::job_description_executable, exe);
    if (!args.empty())
        jd.set_vector_attribute (saga::attributes::job_description_arguments, args);
    jd.set_attribute (saga::attributes::job_description_interactive, "True");

    saga::job::job job = js.create_job (jd);
    job.run ();

    saga::job::state state = job.get_state ();

    if ( state != saga::job::Running )
    {
      std::cerr << "job submission failed: " << state <<  std::endl;
    }
    else
    {
      std::cout << "Job ID: " << job.get_job_id () << std::endl;
      out = job.get_stdout();
    }

    while ( true ) 
    {
      char buffer[255];
      out.read (buffer, sizeof (buffer));
      
      if ( out.gcount () > 0 )
      {
        std::cout << std::string (buffer, out.gcount ());
      }

      if ( out.fail () ) 
      {
        break;
      }
    }

    std::cout << std::endl << std::flush;

    saga::job::job job2 = js.get_job (job.get_job_id ());

    state = job2.get_state ();

    if ( state != saga::job::Running )
    {
      std::cerr << "job submission failed: " << state <<  std::endl;
    }
    else
    {
      std::cout << job2.get_job_id () << std::endl;
    }

    while ( state == saga::job::Running )
    {
      std::cout << "Job state is: " << state << std::endl;
      state = job.get_state ();
      sleep (1);
    }
  }
  catch ( saga::exception const & e ) 
  {
    std::cerr << "saga exception caught: " << e.what () << std::endl;
  }

  return 0;
}

