//  Copyright (c) 2005-2007 Hartmut Kaiser
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <saga.hpp>

///////////////////////////////////////////////////////////////////////////////
int main (int argc, char* argv[])
{
    try {
        namespace attribs = saga::attributes;

        // argv[1] is the binary to launch
        saga::job_description jd;
        jd.set_attribute (attribs::job_description_executable, argv[1]);
        
        // the remaining arguments are interpreted as file transfer specifications
        std::vector<std::string> arguments;
        for (int i = 2; i < argc; ++i)
            arguments.push_back(argv[i]);
        if (!arguments.empty())
            jd.set_vector_attribute (attribs::job_description_filetransfer, arguments);

        // spawn the job
        saga::job_service js("fork://localhost");
        saga::job j = js.create_job(jd);
        j.run();

        // wait for this job to finish
        while (j.get_state() == saga::job::Running)
            ;
    }        
    catch (saga::exception const& e) {
        std::cerr << "job_run_staging: saga::exception caught: " << e.what() << std::endl;
        return -1;
    }
    catch (std::exception const& e) {
        std::cerr << "job_run_staging: exception caught: " << e.what() << std::endl;
        return -1;
    }
    catch (...) {
        std::cerr << "job_run_staging: unexpected exception caught" << std::endl;
        return -1;
    }
    return 0;
}

