//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <saga.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace 
{
    struct print_entries
    {
        print_entries(saga::logical_directory lfd) : lfd_(lfd) {}
        
        template <typename T>
        void operator()(T const& key) const
        {
            if (lfd_.is_entry(key))
                std::cout << "  " << key << std::endl;
            else if (lfd_.is_dir(key))
                std::cout << "d " << key << std::endl;
            else if (lfd_.is_link(key))
                std::cout << "l " << key << std::endl;
            else
                std::cout << "u " << key << std::endl;
        }
        
        saga::logical_directory lfd_;
    };
}

///////////////////////////////////////////////////////////////////////////////
int main (int argc, char* argv[])
{
    if (argc != 3 && argc != 4) {
        std::cout << "Usage: logicaldirectory_list [-r] <lfn> <pattern>" << std::endl;
        exit(-1);
    }
    
    bool recursive = false;
    if (argc == 4) {
        if (std::string(argv[1]) != "-r") {
            std::cout << "Usage: logicaldirectory_list [-r] <lfn> <pattern>" << std::endl;
            exit(-1);
        }
        recursive = true;
    }
    
    using namespace saga;
    session s;
    try {
    // create a UserPass context and associate it with the session
        context ctx(context::UserPass);
        
        ctx.set_attribute(attributes::context_user_name, "SAGA");
        ctx.set_attribute(attributes::context_user_pass, "SAGA_client");
        s.add_context(ctx);
        
    // list the existing lfn's in the given node
        logical_directory lfd(s, argv[(argc == 4) ? 2 : 1]);
        std::vector<std::string> lfns(lfd.list(argv[(argc == 4) ? 3 : 2], 
            recursive ? file::Recursive : file::None));

        if (!lfns.empty()) {
            std::cout << "Existing replicas for lfn '" << lfd.get_url() 
                      << "':" << std::endl;

            std::for_each(lfns.begin(), lfns.end(), print_entries(lfd));
        }
        else {
            std::cout << "No matching replicas exist for lfn '" << lfd.get_url() 
                      << "'" << std::endl;
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

