//  Copyright (c) 2005-2006 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <boost/lambda/lambda.hpp>

#include <saga.hpp>

bool called_on_modified = false;

///////////////////////////////////////////////////////////////////////////////
bool on_modified(saga::object lf, saga::metric m, saga::context ctx)
{
    called_on_modified = true;
    return true;
}

///////////////////////////////////////////////////////////////////////////////
int main (int argc, char* argv[])
{
    if (argc != 3) {
        std::cout << "Usage: logical_file_add_location <lfn> <replica>" << std::endl;
        exit(-1);
    }
    
    using namespace saga;
    session s;
    try {
    // create a UserPass context and associate it with the session
        context ctx(context::UserPass);
        
        ctx.set_attribute(attributes::context_user_name, "SAGA");
        ctx.set_attribute(attributes::context_user_pass, "SAGA_client");
        s.add_context(ctx);
        
    // create a new lfn
        logical_file lf(s, argv[1], logical_file::Create | 
                                    logical_file::CreateParents |
                                    logical_file::ReadWrite);

        saga::monitorable::cookie_handle h = 0;
        
    // look, if the entry already exists
        std::vector<std::string> locs(lf.list_locations());
        if (locs.end() == std::find(locs.begin(), locs.end(), argv[2])) {
        // entry doesn't exist, register callback and add it to the replica
            lf.add_callback(saga::metrics::logical_file_modified, on_modified);
            lf.add_location(argv[2]);
        }
        
    // now read the new lfn entry    
        std::vector<std::string> locations(lf.list_locations());
        if (!locations.empty()) {
            std::cout << "Replica entries for lfn '" << lf.get_url() << "':" 
                      << std::endl;

            using boost::lambda::_1;
            std::for_each(locations.begin(), locations.end(), 
                std::cout << _1 << '\n');
        }
        else {
            std::cout << "No replica entries found for lfn '" << lf.get_url() 
                      << "'" << std::endl;
        }
        
        if (0 != h) {
        // wait for the callback
            while (!called_on_modified)
                ;
            lf.remove_callback(saga::metrics::logical_file_modified, h);
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

