//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <boost/lambda/lambda.hpp>

#include <saga.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace 
{
    struct print_attributes
    {
        print_attributes(saga::logical_file lf) : lf_(lf) {}
        
        template <typename T>
        void operator()(T const& key) const
        {
            std::cout << key << ": " << lf_.get_attribute(key) << std::endl;
        }
        
        saga::logical_file lf_;
    };
}

int main (int argc, char* argv[])
{
    if (argc != 3 && argc != 4) {
        std::cout << "Usage: logical_file_find_attribute <lfn> <kpat> [<vpat>]" 
                  << std::endl;
        exit(-1);
    }
    
    try {
    // open an existing lfn and look for matching attributes
        using namespace saga;
        std::string lfn(argv[1]);
        
        std::string pattern(argv[2]);
        if (4 == argc) 
            pattern += std::string("=") + argv[3];
        else
            pattern += "=*";
        
        logical_file lf(lfn);
        std::vector<std::string> keys = lf.find_attributes(pattern);

    // now list the found attributes
        if (!keys.empty()) {
            std::cout << "Attributes for lfn '" << lf.get_url() << "':" 
                      << std::endl;
            std::for_each(keys.begin(), keys.end(), print_attributes(lf));
        }
        else {
            std::cout << "No attributes found for lfn: '" << lf.get_url() 
                      << "'" << std::endl;
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

