//  Copyright (c) 2005-2006 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <boost/lambda/lambda.hpp>

#include <saga.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace 
{
    struct print_attributes
    {
        print_attributes(saga::logical_file lf) : lf_(lf) {}
        
        template <typename T>
        void operator()(T const& key) const
        {
            std::cout << key << ": " << lf_.get_attribute(key) << std::endl;
        }
        
        saga::logical_file lf_;
    };
}

///////////////////////////////////////////////////////////////////////////////
int main (int argc, char* argv[])
{
    if (argc != 2) {
        std::cout << "Usage: logical_file_list_locations <lfn>" << std::endl;
        exit(-1);
    }
    
    try {
        // instantiate logical file 
        saga::logical_file lf(argv[1]);

        // read replica entries
        std::vector<std::string> locations(lf.list_locations());
        if (!locations.empty()) {
            std::cout << "Replica entries for lfn '" << lf.get_url() << "':" 
                      << std::endl;

            using boost::lambda::_1;
            std::for_each(locations.begin(), locations.end(), 
                std::cout << _1 << '\n');
            std::cout << std::endl;
        }
        else {
            std::cout << "No replica entries found for lfn: '" << lf.get_url() 
                      << "'" << std::endl << std::endl;
        }
        
        // list attributes
        std::vector<std::string> keys(lf.list_attributes());
        if (!keys.empty()) {
            std::cout << "Attributes for lfn '" << lf.get_url() << "':" 
                      << std::endl;
            std::for_each(keys.begin(), keys.end(), print_attributes(lf));
        }
        else {
            std::cout << "No attributes found for lfn: '" << lf.get_url() 
                      << "'" << std::endl;
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

