//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <boost/lambda/lambda.hpp>

#include <saga.hpp>

int main (int argc, char* argv[])
{
    if (argc != 2) {
        std::cout << "Usage: logical_file_remove <lfn>" << std::endl;
        exit(-1);
    }
    
    try {
    // open an existing lfn and remove the given replica
        using namespace saga;
        std::string lfn(argv[1]);
        
        logical_file lf(lfn, logical_file::Read | logical_file::Write);
        lf.remove();

    // now list the existing (remaining) lfn's in the parent node
        std::string::size_type p = lfn.find_last_of('/');
        if (p != std::string::npos)
            lfn = lfn.substr(0, p+1);   // chop of last part of the lfn

        logical_directory lfd(lfn);
        std::vector<std::string> lfns(lfd.list("*"));
        if (!lfns.empty()) {
            std::cout << "Remaining replicas for lfn '" << lfd.get_url() 
                      << "':" << std::endl;

            using boost::lambda::_1;
            std::for_each(lfns.begin(), lfns.end(), std::cout << _1 << '\n');
        }
        else {
            std::cout << "No replicas left for lfn '" << lfd.get_url() 
                      << "'" << std::endl;
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

