//  Copyright (c) 2005-2006 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <saga.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace 
{
    struct print_attributes
    {
        print_attributes(saga::logical_file lf) : lf_(lf) {}
        
        template <typename T>
        void operator()(T const& key) const
        {
            std::cout << key << ": " << lf_.get_attribute(key) << std::endl;
        }
        
        saga::logical_file lf_;
    };
}

int main (int argc, char* argv[])
{
    if (argc != 3) {
        std::cout << "Usage: logical_file_remove_attribute <lfn> <metakey>" 
                  << std::endl;
        exit(-1);
    }
    
    try {
    // open existing/create a new lfn
        using namespace saga;
        logical_file lf(argv[1], logical_file::Read | logical_file::Write);
        lf.remove_attribute(argv[2]);

    // now read all attributes  
        std::vector<std::string> keys(lf.list_attributes());
        if (!keys.empty()) {
            std::cout << "Attributes for lfn '" << lf.get_url() << "':" 
                      << std::endl;
            std::for_each(keys.begin(), keys.end(), print_attributes(lf));
        }
        else {
            std::cout << "No attributes found for lfn: '" << lf.get_url() 
                      << "'" << std::endl;
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

