//  Copyright (c) 2005-2006 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <vector>
#include <algorithm>

#include <boost/lambda/lambda.hpp>

#include <saga.hpp>


int main (int argc, char* argv[])
{
    if (argc != 3) {
        std::cout << "Usage: logical_file_remove_location <lfn> <replica>" << std::endl;
        exit(-1);
    }
    
    try {
    // open an existing lfn and remove the given replica
        using namespace saga;
        logical_file lf(argv[1], logical_file::Read | logical_file::Write);
        lf.remove_location(argv[2]);

    // now read the existing (remaining) lfn entries
        std::vector<std::string> locations(lf.list_locations());
        if (!locations.empty()) {
            std::cout << "Remaining replica entries for lfn '" << lf.get_url() 
                      << "':" << std::endl;

            using boost::lambda::_1;
            std::for_each(locations.begin(), locations.end(), 
                std::cout << _1 << '\n');
        }
        else {
            std::cout << "No replica entries left for lfn '" << lf.get_url() 
                      << "'" << std::endl;
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

