//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <saga.hpp>

void allocate_buffer(saga::rpc::parameter& param, std::size_t size)
{
    param.buffer = saga::buffer(new char[size], size);
}

int main(int argc, char* argv[])
{
    using namespace saga;
    try {
        {
            rpc hello("any://localhost:6789/Hello");
            std::vector<saga::rpc::parameter> args;
            hello.call(args);
        }
        {        
            rpc bin("any://localhost:7890/TestBase64");
            std::vector<saga::rpc::parameter> args(1);
            args[0].mode = saga::rpc::Out;
            args[0].alloc = allocate_buffer;
            bin.call(args);
            delete saga::buffer_cast<char*>(args[0].buffer);
        }
    }
    catch (saga::exception const& e) {
        std::cerr << "saga exception caught: " << e.what () << std::endl;
    }
    catch (std::exception const& e) {
        std::cerr << "exception caught: " << e.what() << std::endl;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
    }
    return 0;
}

