//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <iostream>
#include <saga.hpp>

int main(int argc, char* argv[])
{
    if (argc > 3) {
        std::cerr << "Usage: stream_server [host [port]]" << std::endl;
        std::cerr << "Default host is 'any://localhost', default port is none." 
                  << std::endl;
        return -1;
    }

    try {
    // retrieve parameter values
        std::string host;
        std::string port;
        
        if (argc > 1)
            host = argv[1];
        else
            host = "any://localhost";
            
        if (argc > 2)
            port = argv[2];

    // actual functionality
        std::string url (host);
        if (!port.empty())
            host += ":" + port;
            
        std::cout << "Serving " << (url) << std::endl;
        
        saga::stream_service service(url);
        while(1) {
            saga::stream strm = service.serve();
            std::cout << "Established connection from: " 
              << strm.get_url() << "!" << std::endl;

            strm.write (17, saga::buffer ("test from server\n", 17));
        }
    } 
    catch (saga::exception const& e) {
        std::cerr << "saga::exception caught: " << e.what () << std::endl;
        return -3;
    }
    catch (std::exception const& e) {
        std::cerr << "std::exception caught: " << e.what () << std::endl;
        return -2;
    }
    catch (...) {
        std::cerr << "unexpected exception caught" << std::endl;
        return -1;
    }
    return 0;
}
