#  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
# 
#  Distributed under the Boost Software License, Version 1.0. (See accompanying
#  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

ifndef SAGA_MK_INC_CONF_C
       SAGA_MK_INC_CONF_C = yes

###########################################################
#
# paths
#
SAGA_LOCATION              = /usr

SAGA_BIN_ROOT = $(SAGA_LOCATION)/bin
SAGA_LIB_ROOT = $(SAGA_LOCATION)/lib
SAGA_INI_ROOT = $(SAGA_LOCATION)/share/saga
SAGA_HDR_ROOT = $(SAGA_LOCATION)/include
SAGA_MAK_ROOT = $(SAGA_LOCATION)/share/saga/make
SAGA_EXT_ROOT = $(SAGA_ROOT)/external
#
###########################################################

###########################################################
#
# Tools
#
CP                         = /bin/cp -f
LN                         = /bin/ln -s
LS                         = /bin/ls
MV                         = /bin/mv -f
RM                         = /bin/rm -f
TR                         = /usr/bin/tr
AWK                        = /bin/awk
CAT                        = /bin/cat
CUT                        = /bin/cut
ENV                        = /bin/env
RPM                        = /usr/bin/rpmbuild
PWD                        = /bin/pwd
SED                        = /bin/sed
TAR                        = /bin/tar
DATE                       = /bin/date
GREP                       = /bin/grep
GZIP                       = /bin/gzip
ECHO                       = /bin/echo
FIND                       = /bin/find
GZIP                       = /bin/gzip
TEST                       = /usr/bin/test
TRUE                       = /bin/true
MKDIR                      = /bin/mkdir -p
TOUCH                      = /bin/touch
FALSE                      = /bin/false

MAKE                      += --no-print-directory
#
############################################################

###########################################################
#
# Compiler
#
SAGA_CPP                   = i586-mandriva-linux-gnu-g++
SAGA_CC                    = i586-mandriva-linux-gnu-gcc
SAGA_CXX                   = i586-mandriva-linux-gnu-g++
SAGA_LD                    = i586-mandriva-linux-gnu-g++
SAGA_LD_BIN                = i586-mandriva-linux-gnu-g++
SAGA_LD_LIB                = i586-mandriva-linux-gnu-g++

SAGA_CPPFLAGS              = 
SAGA_CCFLAGS               = -O2 -g -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector --param=ssp-buffer-size=4 -fexceptions -fomit-frame-pointer -march=i586 -mtune=generic -fasynchronous-unwind-tables
SAGA_CXXFLAGS              = -O2 -g -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector --param=ssp-buffer-size=4 -fexceptions -fomit-frame-pointer -march=i586 -mtune=generic -fasynchronous-unwind-tables
SAGA_LDFLAGS_EXT           =  -ldl

SAGE_LEXT                  = so

SAGA_CCFLAGS              += $(SAGA_INC_FLAGS)
SAGA_CXXFLAGS             += $(SAGA_INC_FLAGS)
SAGA_LDFLAGS_EXT          += $(SAGA_LIB_FLAGS)
#
###########################################################


SAGA_OSTYPE                = $(shell uname | $(TR) 'a-z' 'A-Z')

###########################################################
#
# MacOS X specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "DARWIN"

  SAGA_CXXFLAGS           += -x c++ -pipe -Os 
  SAGA_CXXFLAGS           += -Wno-trigraphs -fvisibility-inlines-hidden
  SAGA_LD_LIB             += -dynamiclib
  SAGA_LEXT                = dylib

  SAGA_CCFLAGS            += -D__DARWIN__ -D_FILE_OFFSET_BITS=64
  SAGA_CXXFLAGS           += -D__DARWIN__ -D_FILE_OFFSET_BITS=64

endif
#
###########################################################

###########################################################
#
# IBM AIX 5 specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "AIX"

  SAGA_LEXT                  = so
  SAGA_CCFLAGS              += -D__AIX__ 
  SAGA_CXXFLAGS             += -D__AIX__ 

  ifeq "$(SAGA_CC)" "gcc"  
    
    SAGA_LD_LIB             += -shared
    SAGA_LDFLAGS            += -pthread -shared -Wl,-G -Wl,-brtl -Wl,-binitfini:_GLOBAL__DI:_GLOBAL__DD
  
  endif
   
  ifeq "$(SAGA_CC)" "xlc"  
   
    SAGA_LDFLAGS           += -L/usr/lib/threads -G -brtl -bnoentry -lpthread -lc_r
    SAGA_CCFLAGS           += -bnoentry -D_THREAD_SAFE
    SAGA_CXXFLAGS          += -bnoentry -D_THREAD_SAFE
  
  endif
   
endif
#
###########################################################

###########################################################
# 
# Linux specific compiler settings
#
ifeq "$(SAGA_OSTYPE)" "LINUX"

  SAGA_LEXT                = so
  SAGA_LD_LIB             += -shared

  SAGA_CCFLAGS            += -D__LINUX__ -D_FILE_OFFSET_BITS=64
  SAGA_CXXFLAGS           += -D__LINUX__ -D_FILE_OFFSET_BITS=64

  ifeq "$(SAGA_CC)" "gcc"
    SAGA_CCFLAGS          += -pipe -pthread -fPIC -fvisibility=default
  endif
  ifeq "$(SAGA_CC)" "icc"
    SAGA_CCFLAGS          += -cxxlib-icc -openmp 
  endif

  SAGA_CCFLAGS            += -Wall -Wno-unknown-pragmas

  ifeq "$(SAGA_CXX)" "g++"
    SAGA_CXXFLAGS         += $(SAGA_CCFLAGS)
    SAGA_CXXFLAGS         += -pipe -pthread -fPIC -fvisibility=default
  endif
  ifeq "$(SAGA_CXX)" "icc"
    SAGA_CXXFLAGS         += $(SAGA_CCFLAGS)
  endif

endif
#
###########################################################

###########################################################
#
# well, we always need boost...
#
ifeq "$(filter clean distclean confclean,$(MAKECMDGOALS))" ""
  ifdef SAGA_ROOT
    include $(SAGA_ROOT)/make/saga.config.boost.c.mk
  else
    include $(SAGA_LOCATION)/share/saga/make/saga.config.boost.c.mk
  endif
endif
#
###########################################################


###########################################################
#
# we also load any configuration checks we may know about
#
ifeq "$(filter clean distclean confclean,$(MAKECMDGOALS))" ""
  ifdef SAGA_ROOT
    include $(SAGA_ROOT)/make/saga.config.check.c.mk
  else
    include $(SAGA_LOCATION)/share/saga/make/saga.config.check.c.mk
  endif
endif
#
###########################################################

endif # guard

